#ifndef FuzzyWorkerH
#define FuzzyWorkerH
#include "GCP.H"
#include "GUI_Checkpoint.H"
class FuzzyTriangle;


struct Rule
{
	GCP_Label* label;
	GCP_Button* deleteb;
	FuzzyTriangle* IF1;
	FuzzyTriangle* IF2;
	FuzzyTriangle* RE;
};

struct stCar
{
	double dEnergy, dMass, dSpeed, dDistance;
	double numDistance;
	double numEnergy;
	double numSpeed;

	double distanceScale;
	double energyScale;
};		

struct simulationResult
{
	string status;
	int value;
};

struct opts
{
	int iSpeciesNum;
	int iChromosomeNum;
	double dMutationProbability;
};



class FuzzyWorker
{
	private:
		GCP_Controller* Interface;
		GCP_Form* MainInterfaceForm;
		GCP_Button* Car;
		GCP_Button* bStartWall, *bEndWall;
		stCar CarInfo;
		GCP_CheckBox* bCheckbox;
		

		FuzzyTriangle* FZDistance_trinagleLow;
		FuzzyTriangle* FZDistance_trinagleMedium;
		FuzzyTriangle* FZDistance_trinagleHigh;

		FuzzyTriangle* FZEnergy_trinagleLow;
		FuzzyTriangle* FZEnergy_trinagleMedium;
		FuzzyTriangle* FZEnergy_trinagleHigh;

		FuzzyTriangle* FZStop_trinagleLow;
		FuzzyTriangle* FZStop_trinagleMedium;
		FuzzyTriangle* FZStop_trinagleHigh;

		GCP_Form* OptionsForm;
		GCP_Edit* Edit;
		GCP_Edit* Edit2;
		GCP_Edit* Edit3;

		GCP_Select* selectIf2, *selectIf1, *selectResult;
		GCP_Vector<Rule> vectorRules;
		
		bool isContinueEducation;
		opts opt;

		

	public:
		double FuzzyOperation(double iSpeed, double iDistance);
		void FuzzyStartSimulation(void* obj);
		void FuzzyStartEducation(void* obj);	
		void FuzzyContinueEducation(void* obj);		
		void initGUI();
		void draw();
		simulationResult drawSimulation();
		void LockOrdinates(void* obj);
		void OnAddRule(void* obj);
		void OnRuleDelete(void *obj);
		void defaultRules(void* obj);
		void OnOptonsFormOkClick(void* obj);
		void OnOptonsFormOpen(void* obj);
		void FuzzyWorker::deleteRules();
		int setPositionFromBinParametrs(string parametrs);
		void* SimulationFunc(void *obj);
		int update();
};

class FuzzyTriangle
{
	private:
		GUI_Checkpoint *bFuzzyLow;
		GUI_Checkpoint *bFuzzyLowOne;
		GUI_Checkpoint *bFuzzyLowTwo;
		GCP_Color c;

		int minX, maxX;
		int iHeigth;
		GCP_Label *label;
		
	public: 
		void setCaption(string s)
		{
			label->text  = s;
		}
		FuzzyTriangle(GCP_Form* form, int topX, int topY, int height, GCP_Color color)
		{
			c = color;
			bFuzzyLow = new GUI_Checkpoint(topX,topY,5,c,true);
			bFuzzyLow->setOnDrag(this,&FuzzyTriangle::LockOrdinates);

			bFuzzyLowOne = new GUI_Checkpoint(topX-50,topY+height,5,c,true);
			bFuzzyLowOne->setOnDrag(this,&FuzzyTriangle::LockOrdinates);

			bFuzzyLowTwo = new GUI_Checkpoint(topX+50,topY+height,5,c,true);
			bFuzzyLowTwo->setOnDrag(this,&FuzzyTriangle::LockOrdinates);

			label = new GCP_Label();
			label->setWidthHeight(150,25);
			label->setColor(c_white,c_black,c_white,c_black);
			

			form->addComponent(label);
			form->addComponent(bFuzzyLow);
			form->addComponent(bFuzzyLowOne);
			form->addComponent(bFuzzyLowTwo);
			label->setPosition(bFuzzyLow->xPos-45,bFuzzyLow->yPos+5);
			iHeigth = height;
		}

		GCP_Vector<int>* getCoordinates()
		{
			GCP_Vector<int> *position = new GCP_Vector<int> ();
			position->push_back(bFuzzyLow->xPos);
			//position->push_back(bFuzzyLow->yPos);

			position->push_back(bFuzzyLowOne->xPos);
			//position->push_back(bFuzzyLowOne->yPos);

			//if(bFuzzyLowTwo->isVisible){
			position->push_back(bFuzzyLowTwo->xPos);
			//position->push_back(bFuzzyLowTwo->yPos);
			//}

			
			return position;				
		}

		void setCoordinates(GCP_Vector<int> *position)
		{
			//int p = position->size();
			bFuzzyLow->setPosition(position->pop_first(),bFuzzyLow->yPos);
			bFuzzyLowOne->setPosition(position->pop_first(),bFuzzyLowOne->yPos);
			//if(position.size())
			bFuzzyLowTwo->setPosition(position->pop_first(),bFuzzyLowTwo->yPos);
			
			LockOrdinates((void*)bFuzzyLow);
		}

		void setOnlyOneLine()
		{
			bFuzzyLowTwo->isVisible = false;
		}
		void setRange(int minR, int maxR)
		{
			minX = minR;
			maxX = maxR;
		}

		double getValue(double position, bool determinedval = false)
		{
			//  
			if((position<minX || position>maxX)&& !determinedval)
				return 0;

			GCP_Line X;
			GCP_Line A (bFuzzyLowOne->xPos,bFuzzyLowOne->yPos,bFuzzyLow->xPos,bFuzzyLow->yPos);
			GCP_Line B (bFuzzyLowTwo->xPos,bFuzzyLowTwo->yPos,bFuzzyLow->xPos,bFuzzyLow->yPos);
			GCP_Point Intersection;

			if(determinedval && (position > 0 &&  position < 100))
			{
				X.pointA = GCP_Point((double)minX,bFuzzyLowOne->yPos-iHeigth*position/100);
				X.pointB = GCP_Point((double)maxX,bFuzzyLowOne->yPos-iHeigth*position/100);
			}else
			{
				X.pointA = GCP_Point(position,(double)bFuzzyLow->yPos);
				X.pointB = GCP_Point(position,(double)bFuzzyLow->yPos+iHeigth);
			}

		


			


			//    
			bool isIntersect = GCP_Math::lineIntersection(X,A,&Intersection,false);
			if(isIntersect)
				isIntersect  = (GCP_Math::isPointInLine(A,Intersection)&&GCP_Math::isPointInLine(X,Intersection));

			if(!isIntersect && bFuzzyLowTwo->isVisible  && !determinedval)
			{
				//    
				isIntersect = GCP_Math::lineIntersection(X,B,&Intersection,false);
				isIntersect  =(GCP_Math::isPointInLine(B,Intersection)&&GCP_Math::isPointInLine(X,Intersection));
			}

			if(isIntersect && determinedval)
			{
				double dIntersX = Intersection.X;
				return dIntersX;
			}

			if(isIntersect && !determinedval)
			{
				//max = bFuzzyLow->yPos   min - bFuzzyLow->yPos+iHeigth
				double dIntersY = Intersection.Y;
				dIntersY -= bFuzzyLow->yPos;

				//iHeigth - 100%   dIntersY - ?
				double ret = 100-(double)(dIntersY*100)/iHeigth;
				return  100-(double)(dIntersY*100)/iHeigth;
			}			
			
			return 0;
		}
					
		void LockOrdinates(void *obj)
		{
			GCP_Button * button = (GCP_Button*)obj;
			if(button!= 0)
			{
				button->yPos = button->yPosStart;

				//    
				bFuzzyLow->xPos = min(max(bFuzzyLow->xPos, minX),maxX);
				bFuzzyLowOne->xPos = min(max(bFuzzyLowOne->xPos, minX),maxX);
				bFuzzyLowTwo->xPos = min(max(bFuzzyLowTwo->xPos, minX),maxX);

				//    
				if(bFuzzyLowTwo->isVisible){
				if(bFuzzyLow->xPos > bFuzzyLowOne->xPos)
					if(bFuzzyLow->xPos > bFuzzyLowTwo->xPos)
					{
						if(bFuzzyLowOne->xPos > bFuzzyLowTwo->xPos)
							bFuzzyLowOne->xPos = bFuzzyLow->xPos;
						else bFuzzyLowTwo->xPos = bFuzzyLow->xPos;
					}

				if(bFuzzyLow->xPos < bFuzzyLowOne->xPos)
					if(bFuzzyLow->xPos < bFuzzyLowTwo->xPos)
					{
						if(bFuzzyLowOne->xPos < bFuzzyLowTwo->xPos)
							bFuzzyLowOne->xPos = bFuzzyLow->xPos;
						else bFuzzyLowTwo->xPos = bFuzzyLow->xPos;
					}
				}
				label->setPosition(bFuzzyLow->xPos-45,bFuzzyLow->yPos+5);
			}
		}

		void draw(SDL_Renderer* sdlRenderer)
		{

			GCP_Draw::Draw_Line(sdlRenderer, bFuzzyLow->xPos,bFuzzyLow->yPos,
				bFuzzyLowOne->xPos, bFuzzyLowOne->yPos,c);

			if(bFuzzyLowTwo->isVisible)
			GCP_Draw::Draw_Line(sdlRenderer, bFuzzyLow->xPos,bFuzzyLow->yPos,
				bFuzzyLowTwo->xPos, bFuzzyLowTwo->yPos,c);

		}
};
#endif